@@ -18,6 +18,7 @@ use async_trait::async_trait;
 #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
 use collector::CollectorAsyncClientHttp;
 use opentelemetry_semantic_conventions as semcov;
+use std::convert::TryInto;
 
 #[cfg(feature = "isahc_collector_client")]
 #[allow(unused_imports)] // this is actually used to configure authentication
@@ -33,6 +34,7 @@ use opentelemetry::{
 };
 #[cfg(feature = "collector_client")]
 use opentelemetry_http::HttpClient;
+use std::collections::HashSet;
 use std::{
     net,
     time::{Duration, SystemTime},
@@ -48,8 +50,6 @@ use uploader::{AsyncUploader, SyncUploader, Uploader};
     not(feature = "isahc_collector_client")
 ))]
 use headers::authorization::Credentials;
-use opentelemetry::sdk::resource::ResourceDetector;
-use opentelemetry::sdk::resource::SdkProvidedResourceDetector;
 use opentelemetry::sdk::trace::Config;
 use opentelemetry::sdk::Resource;
 use std::sync::Arc;
@@ -91,18 +91,9 @@ impl trace::SpanExporter for Exporter {
     /// Export spans to Jaeger
     async fn export(&mut self, batch: Vec<trace::SpanData>) -> trace::ExportResult {
         let mut jaeger_spans: Vec<jaeger::Span> = Vec::with_capacity(batch.len());
-        let mut process = self.process.clone();
+        let process = self.process.clone();
 
-        for (idx, span) in batch.into_iter().enumerate() {
-            if idx == 0 {
-                if let Some(span_process_tags) = build_process_tags(&span) {
-                    if let Some(process_tags) = &mut process.tags {
-                        process_tags.extend(span_process_tags);
-                    } else {
-                        process.tags = Some(span_process_tags.collect())
-                    }
-                }
-            }
+        for span in batch.into_iter() {
             jaeger_spans.push(convert_otel_span_into_jaeger_span(
                 span,
                 self.export_instrumentation_lib,
@@ -131,6 +122,7 @@ pub struct PipelineBuilder {
     service_name: Option<String>,
     tags: Option<Vec<KeyValue>>,
     max_packet_size: Option<usize>,
+    auto_split: bool,
     config: Option<sdk::trace::Config>,
 }
 
@@ -151,6 +143,7 @@ impl Default for PipelineBuilder {
             service_name: None,
             tags: None,
             max_packet_size: None,
+            auto_split: false,
             config: None,
         };
 
@@ -214,6 +207,20 @@ impl PipelineBuilder {
         }
     }
 
+    /// Get collector's username set in the builder. Default to be the value of
+    /// `OTEL_EXPORTER_JAEGER_USER` environment variable.
+    ///
+    /// If users uses custom http client. This function can help retrieve the value of
+    /// `OTEL_EXPORTER_JAEGER_USER` environment variable.
+    #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
+    #[cfg_attr(
+        docsrs,
+        doc(any(feature = "collector_client", feature = "wasm_collector_client"))
+    )]
+    pub fn collector_username(&self) -> Option<String> {
+        (&self.collector_username).clone()
+    }
+
     /// Assign the collector password
     #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
     #[cfg_attr(
@@ -227,13 +234,36 @@ impl PipelineBuilder {
         }
     }
 
+    /// Get the collector's password set in the builder. Default to be the value of
+    /// `OTEL_EXPORTER_JAEGER_PASSWORD` environment variable.
+    ///
+    /// If users uses custom http client. This function can help retrieve the value of
+    /// `OTEL_EXPORTER_JAEGER_PASSWORD` environment variable.
+    #[cfg(any(feature = "collector_client", feature = "wasm_collector_client"))]
+    #[cfg_attr(
+        docsrs,
+        doc(any(feature = "collector_client", feature = "wasm_collector_client"))
+    )]
+    pub fn collector_password(self) -> Option<String> {
+        (&self.collector_password).clone()
+    }
+
     /// Assign the process service name.
     pub fn with_service_name<T: Into<String>>(mut self, service_name: T) -> Self {
         self.service_name = Some(service_name.into());
         self
     }
 
-    /// Assign the process service tags.
+    /// Assign the process tags.
+    ///
+    /// Note that resource in trace [Config](sdk::trace::Config) is also reported as process tags
+    /// in jaeger. If there is duplicate tags between resource and tags. Resource's value take
+    /// priority even if it's empty.
+    #[deprecated(
+        since = "0.16.0",
+        note = "please pass those tags as resource in sdk::trace::Config. Then use with_trace_config \
+        method to pass the config. All key value pairs in resources will be reported as process tags"
+    )]
     pub fn with_tags<T: IntoIterator<Item = KeyValue>>(mut self, tags: T) -> Self {
         self.tags = Some(tags.into_iter().collect());
         self
@@ -245,7 +275,37 @@ impl PipelineBuilder {
         self
     }
 
+    /// Config whether to auto split batches.
+    ///
+    /// When auto split is set to true, the exporter will try to split the
+    /// batch into smaller ones so that there will be minimal data loss. It
+    /// will impact the performance.
+    ///
+    /// Note that if one span is too large to export, other spans within the
+    /// same batch may or may not be exported. In this case, exporter will
+    /// return errors as we cannot split spans.
+    pub fn with_auto_split_batch(mut self, auto_split: bool) -> Self {
+        self.auto_split = auto_split;
+        self
+    }
+
     /// Assign the SDK config for the exporter pipeline.
+    ///
+    /// # Examples
+    /// Set service name via resource.
+    /// ```rust
+    /// use opentelemetry_jaeger::PipelineBuilder;
+    /// use opentelemetry::sdk;
+    /// use opentelemetry::sdk::Resource;
+    /// use opentelemetry::KeyValue;
+    ///
+    /// let pipeline = PipelineBuilder::default()
+    ///                 .with_trace_config(
+    ///                       sdk::trace::Config::default()
+    ///                         .with_resource(Resource::new(vec![KeyValue::new("service.name", "my-service")]))
+    ///                 );
+    ///
+    /// ```
     pub fn with_trace_config(self, config: sdk::trace::Config) -> Self {
         PipelineBuilder {
             config: Some(config),
@@ -263,8 +323,11 @@ impl PipelineBuilder {
     /// Install a Jaeger pipeline with a simple span processor.
     pub fn install_simple(self) -> Result<sdk::trace::Tracer, TraceError> {
         let tracer_provider = self.build_simple()?;
-        let tracer =
-            tracer_provider.tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
+        let tracer = tracer_provider.versioned_tracer(
+            "opentelemetry-jaeger",
+            Some(env!("CARGO_PKG_VERSION")),
+            None,
+        );
         let _ = global::set_tracer_provider(tracer_provider);
         Ok(tracer)
     }
@@ -275,66 +338,75 @@ impl PipelineBuilder {
         runtime: R,
     ) -> Result<sdk::trace::Tracer, TraceError> {
         let tracer_provider = self.build_batch(runtime)?;
-        let tracer =
-            tracer_provider.tracer("opentelemetry-jaeger", Some(env!("CARGO_PKG_VERSION")));
+        let tracer = tracer_provider.versioned_tracer(
+            "opentelemetry-jaeger",
+            Some(env!("CARGO_PKG_VERSION")),
+            None,
+        );
         let _ = global::set_tracer_provider(tracer_provider);
         Ok(tracer)
     }
 
-    // To reduce the overhead of copying service name in every spans. We remove service.name
-    // from the resource in config. Instead, we store it in process.
-    // The service name tag will attch to spans when it's exported.
-    fn build_config_and_process(&mut self) -> (Config, Process) {
-        let service_name = self.service_name.take();
-        if let Some(service_name) = service_name {
-            let config = if let Some(mut cfg) = self.config.take() {
-                cfg.resource = cfg.resource.map(|r| {
-                    let without_service_name = r
-                        .iter()
-                        .filter(|(k, _v)| **k != semcov::resource::SERVICE_NAME)
-                        .map(|(k, v)| KeyValue::new(k.clone(), v.clone()))
-                        .collect::<Vec<KeyValue>>();
-                    Arc::new(Resource::new(without_service_name))
-                });
-                cfg
-            } else {
-                Config {
-                    resource: Some(Arc::new(Resource::empty())),
-                    ..Default::default()
-                }
-            };
-            (
-                config,
-                Process {
-                    service_name,
-                    tags: self.tags.take().unwrap_or_default(),
-                },
-            )
+    // To reduce the overhead of copying service name in every spans. We convert resource into jaeger tags
+    // and store them into process. And set the resource in trace config to empty.
+    //
+    // There are multiple ways to set the service name. A `service.name` tag will be always added
+    // to the process tags.
+    fn build_config_and_process(&mut self, sdk_provided_resource: Resource) -> (Config, Process) {
+        let (config, resource) = if let Some(mut config) = self.config.take() {
+            let resource =
+                if let Some(resource) = config.resource.replace(Arc::new(Resource::empty())) {
+                    sdk_provided_resource.merge(resource)
+                } else {
+                    sdk_provided_resource
+                };
+
+            (config, resource)
         } else {
-            let service_name = SdkProvidedResourceDetector
-                .detect(Duration::from_secs(0))
+            (Config::default(), sdk_provided_resource)
+        };
+
+        let service_name = self.service_name.clone().unwrap_or_else(|| {
+            resource
                 .get(semcov::resource::SERVICE_NAME)
-                .unwrap()
-                .to_string();
-            (
-                Config {
-                    // use a empty resource to prevent TracerProvider to assign a service name.
-                    resource: Some(Arc::new(Resource::empty())),
-                    ..Default::default()
-                },
-                Process {
-                    service_name,
-                    tags: self.tags.take().unwrap_or_default(),
-                },
-            )
+                .map(|v| v.to_string())
+                .unwrap_or_else(|| "unknown_service".to_string())
+        });
+
+        // merge the tags and resource. Resources take priority.
+        let mut tags = resource
+            .into_iter()
+            .filter(|(key, _)| *key != semcov::resource::SERVICE_NAME)
+            .map(|(key, value)| KeyValue::new(key, value))
+            .collect::<Vec<KeyValue>>();
+
+        tags.push(KeyValue::new(
+            semcov::resource::SERVICE_NAME,
+            service_name.clone(),
+        ));
+
+        // if users provide key list
+        if let Some(provided_tags) = self.tags.take() {
+            let key_set: HashSet<Key> = tags
+                .iter()
+                .map(|key_value| key_value.key.clone())
+                .collect::<HashSet<Key>>();
+            for tag in provided_tags.into_iter() {
+                if !key_set.contains(&tag.key) {
+                    tags.push(tag)
+                }
+            }
         }
+
+        (config, Process { service_name, tags })
     }
 
     /// Build a configured `sdk::trace::TracerProvider` with a simple span processor.
     pub fn build_simple(mut self) -> Result<sdk::trace::TracerProvider, TraceError> {
-        let (config, process) = self.build_config_and_process();
+        let mut builder = sdk::trace::TracerProvider::builder();
+        let (config, process) = self.build_config_and_process(builder.sdk_provided_resource());
         let exporter = self.init_sync_exporter_with_process(process)?;
-        let mut builder = sdk::trace::TracerProvider::builder().with_simple_exporter(exporter);
+        builder = builder.with_simple_exporter(exporter);
         builder = builder.with_config(config);
 
         Ok(builder.build())
@@ -346,10 +418,10 @@ impl PipelineBuilder {
         mut self,
         runtime: R,
     ) -> Result<sdk::trace::TracerProvider, TraceError> {
-        let (config, process) = self.build_config_and_process();
+        let mut builder = sdk::trace::TracerProvider::builder();
+        let (config, process) = self.build_config_and_process(builder.sdk_provided_resource());
         let exporter = self.init_async_exporter_with_process(process, runtime.clone())?;
-        let mut builder =
-            sdk::trace::TracerProvider::builder().with_batch_exporter(exporter, runtime);
+        builder = builder.with_batch_exporter(exporter, runtime);
         builder = builder.with_config(config);
 
         Ok(builder.build())
@@ -359,7 +431,8 @@ impl PipelineBuilder {
     ///
     /// This is useful if you are manually constructing a pipeline.
     pub fn init_sync_exporter(mut self) -> Result<Exporter, TraceError> {
-        let (_, process) = self.build_config_and_process();
+        let builder = sdk::trace::TracerProvider::builder();
+        let (_, process) = self.build_config_and_process(builder.sdk_provided_resource());
         self.init_sync_exporter_with_process(process)
     }
 
@@ -381,7 +454,8 @@ impl PipelineBuilder {
         mut self,
         runtime: R,
     ) -> Result<Exporter, TraceError> {
-        let (_, process) = self.build_config_and_process();
+        let builder = sdk::trace::TracerProvider::builder();
+        let (_, process) = self.build_config_and_process(builder.sdk_provided_resource());
         self.init_async_exporter_with_process(process, runtime)
     }
 
@@ -401,9 +475,12 @@ impl PipelineBuilder {
     }
 
     fn init_sync_uploader(self) -> Result<Box<dyn Uploader>, TraceError> {
-        let agent =
-            agent::AgentSyncClientUdp::new(self.agent_endpoint.as_slice(), self.max_packet_size)
-                .map_err::<Error, _>(Into::into)?;
+        let agent = agent::AgentSyncClientUdp::new(
+            self.agent_endpoint.as_slice(),
+            self.max_packet_size,
+            self.auto_split,
+        )
+        .map_err::<Error, _>(Into::into)?;
         Ok(Box::new(SyncUploader::Agent(agent)))
     }
 
@@ -416,6 +493,7 @@ impl PipelineBuilder {
             self.agent_endpoint.as_slice(),
             self.max_packet_size,
             runtime,
+            self.auto_split,
         )
         .map_err::<Error, _>(Into::into)?;
         Ok(Box::new(AsyncUploader::Agent(agent)))
@@ -512,8 +590,9 @@ impl PipelineBuilder {
             Ok(Box::new(uploader))
         } else {
             let endpoint = self.agent_endpoint.as_slice();
-            let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size, runtime)
-                .map_err::<Error, _>(Into::into)?;
+            let agent =
+                AgentAsyncClientUdp::new(endpoint, self.max_packet_size, runtime, self.auto_split)
+                    .map_err::<Error, _>(Into::into)?;
             Ok(Box::new(AsyncUploader::Agent(agent)))
         }
     }
@@ -537,7 +616,7 @@ impl PipelineBuilder {
             Ok(Box::new(AsyncUploader::Collector(collector)))
         } else {
             let endpoint = self.agent_endpoint.as_slice();
-            let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size)
+            let agent = AgentAsyncClientUdp::new(endpoint, self.max_packet_size, self.auto_split)
                 .map_err::<Error, _>(Into::into)?;
             Ok(Box::new(AsyncUploader::Agent(agent)))
         }
@@ -568,15 +647,16 @@ fn links_to_references(links: sdk::trace::EvictedQueue<Link>) -> Option<Vec<jaeg
             .iter()
             .map(|link| {
                 let span_context = link.span_context();
-                let trace_id = span_context.trace_id().to_u128();
-                let trace_id_high = (trace_id >> 64) as i64;
-                let trace_id_low = trace_id as i64;
+                let trace_id_bytes = span_context.trace_id().to_bytes();
+                let (high, low) = trace_id_bytes.split_at(8);
+                let trace_id_high = i64::from_be_bytes(high.try_into().unwrap());
+                let trace_id_low = i64::from_be_bytes(low.try_into().unwrap());
 
                 jaeger::SpanRef::new(
                     jaeger::SpanRefType::FollowsFrom,
                     trace_id_low,
                     trace_id_high,
-                    span_context.span_id().to_u64() as i64,
+                    i64::from_be_bytes(span_context.span_id().to_bytes()),
                 )
             })
             .collect();
@@ -591,14 +671,15 @@ fn convert_otel_span_into_jaeger_span(
     span: trace::SpanData,
     export_instrument_lib: bool,
 ) -> jaeger::Span {
-    let trace_id = span.span_context.trace_id().to_u128();
-    let trace_id_high = (trace_id >> 64) as i64;
-    let trace_id_low = trace_id as i64;
+    let trace_id_bytes = span.span_context.trace_id().to_bytes();
+    let (high, low) = trace_id_bytes.split_at(8);
+    let trace_id_high = i64::from_be_bytes(high.try_into().unwrap());
+    let trace_id_low = i64::from_be_bytes(low.try_into().unwrap());
     jaeger::Span {
         trace_id_low,
         trace_id_high,
-        span_id: span.span_context.span_id().to_u64() as i64,
-        parent_span_id: span.parent_span_id.to_u64() as i64,
+        span_id: i64::from_be_bytes(span.span_context.span_id().to_bytes()),
+        parent_span_id: i64::from_be_bytes(span.parent_span_id.to_bytes()),
         operation_name: span.name.into_owned(),
         references: links_to_references(span.links),
         flags: span.span_context.trace_flags().to_u8() as i32,
@@ -627,20 +708,6 @@ fn convert_otel_span_into_jaeger_span(
     }
 }
 
-fn build_process_tags(
-    span_data: &trace::SpanData,
-) -> Option<impl Iterator<Item = jaeger::Tag> + '_> {
-    span_data
-        .resource
-        .as_ref()
-        .filter(|resource| !resource.is_empty())
-        .map(|resource| {
-            resource
-                .iter()
-                .map(|(k, v)| KeyValue::new(k.clone(), v.clone()).into())
-        })
-}
-
 fn build_span_tags(
     attrs: sdk::trace::EvictedHashMap,
     instrumentation_lib: Option<sdk::InstrumentationLibrary>,
@@ -769,7 +836,9 @@ mod collector_client_tests {
     use crate::exporter::thrift::jaeger::Batch;
     use crate::new_pipeline;
     use opentelemetry::runtime::Tokio;
+    use opentelemetry::sdk::Resource;
     use opentelemetry::trace::TraceError;
+    use opentelemetry::KeyValue;
 
     mod test_http_client {
         use async_trait::async_trait;
@@ -799,9 +868,11 @@ mod collector_client_tests {
         let mut builder = new_pipeline()
             .with_collector_endpoint("localhost:6831")
             .with_http_client(test_http_client::TestHttpClient);
-        let (_, process) = builder.build_config_and_process();
+        let sdk_provided_resource =
+            Resource::new(vec![KeyValue::new("service.name", "unknown_service")]);
+        let (_, process) = builder.build_config_and_process(sdk_provided_resource);
         let mut uploader = builder.init_async_uploader(Tokio)?;
-        let res = futures::executor::block_on(async {
+        let res = futures_executor::block_on(async {
             uploader
                 .upload(Batch::new(process.into(), Vec::new()))
                 .await
@@ -844,9 +915,12 @@ mod tests {
     use super::SPAN_KIND;
     use crate::exporter::thrift::jaeger::Tag;
     use crate::exporter::{build_span_tags, OTEL_STATUS_CODE, OTEL_STATUS_DESCRIPTION};
-    use opentelemetry::sdk::trace::EvictedHashMap;
+    use opentelemetry::sdk::trace::{Config, EvictedHashMap};
+    use opentelemetry::sdk::Resource;
     use opentelemetry::trace::{SpanKind, StatusCode};
     use opentelemetry::KeyValue;
+    use std::env;
+    use std::sync::Arc;
 
     fn assert_tag_contains(tags: Vec<Tag>, key: &'static str, expect_val: &'static str) {
         assert_eq!(
@@ -948,4 +1022,59 @@ mod tests {
         assert_tag_contains(tags.clone(), OTEL_STATUS_CODE, user_status_code.as_str());
         assert_tag_contains(tags, OTEL_STATUS_DESCRIPTION, user_status_description);
     }
+
+    #[test]
+    fn test_set_service_name() {
+        let service_name = "halloween_service";
+
+        // set via builder's service name, it has highest priority
+        let mut builder = crate::PipelineBuilder::default();
+        builder = builder.with_service_name(service_name);
+        let (_, process) = builder.build_config_and_process(Resource::empty());
+        assert_eq!(process.service_name, service_name);
+
+        // make sure the tags in resource are moved to process
+        builder = crate::PipelineBuilder::default();
+        builder = builder.with_service_name(service_name);
+        builder = builder.with_trace_config(
+            Config::default()
+                .with_resource(Resource::new(vec![KeyValue::new("test-key", "test-value")])),
+        );
+        let (config, process) = builder.build_config_and_process(Resource::empty());
+        assert_eq!(config.resource, Some(Arc::new(Resource::empty())));
+        assert_eq!(process.tags.len(), 2);
+
+        // sdk provided resource can override service name if users didn't provided service name to builder
+        builder = crate::PipelineBuilder::default();
+        let (_, process) = builder.build_config_and_process(Resource::new(vec![KeyValue::new(
+            "service.name",
+            "halloween_service",
+        )]));
+        assert_eq!(process.service_name, "halloween_service");
+
+        // users can also provided service.name from config's resource, in this case, it will override the
+        // sdk provided service name
+        builder = crate::PipelineBuilder::default();
+        builder = builder.with_trace_config(Config::default().with_resource(Resource::new(vec![
+            KeyValue::new("service.name", "override_service"),
+        ])));
+        let (_, process) = builder.build_config_and_process(Resource::new(vec![KeyValue::new(
+            "service.name",
+            "halloween_service",
+        )]));
+
+        assert_eq!(process.service_name, "override_service");
+        assert_eq!(process.tags.len(), 1);
+        assert_eq!(
+            process.tags[0],
+            KeyValue::new("service.name", "override_service")
+        );
+
+        // OTEL_SERVICE_NAME env var also works
+        env::set_var("OTEL_SERVICE_NAME", "test service");
+        builder = crate::PipelineBuilder::default();
+        let exporter = builder.init_sync_exporter().unwrap();
+        assert_eq!(exporter.process.service_name, "test service");
+        env::set_var("OTEL_SERVICE_NAME", "")
+    }
 }
